@@ -1,16 +1,13 @@
 ###############################################################################
 #
-# This file copyright (c) 2001 by Randy J. Ray <rjray@blackperl.com>,
-# all rights reserved
+# This file copyright (c) 2001-2014 Randy J. Ray, all rights reserved
 #
 # Copying and distribution are permitted under the terms of the Artistic
-# License as distributed with Perl versions 5.002 and later. See
-# http://www.opensource.org/licenses/artistic-license.php
+# License 2.0 (http://www.opensource.org/licenses/artistic-license-2.0.php) or
+# the GNU LGPL (http://www.opensource.org/licenses/lgpl-2.1.php).
 #
 ###############################################################################
 #
-#   $Id: Client.pm,v 1.22 2004/12/09 08:50:17 rjray Exp $
-#
 #   Description:    This class implements an RPC::XML client, using LWP to
 #                   manage the underlying communication protocols. It relies
 #                   on the RPC::XML transaction core for data management.
@@ -26,6 +23,8 @@
 #                   HTTP::Request
 #                   URI
 #                   RPC::XML
+#                   RPC::XML::ParserFactory
+#                   Compress::Raw::Zlib is used if available
 #
 #   Global Consts:  $VERSION
 #
@@ -33,20 +32,34 @@
 
 package RPC::XML::Client;
 
-use 5.005;
+use 5.008008;
 use strict;
-use vars qw($VERSION);
+use warnings;
+use vars qw($VERSION $COMPRESSION_AVAILABLE);
 use subs qw(new simple_request send_request uri useragent request
-            fault_handler error_handler combined_handler);
+            fault_handler error_handler combined_handler timeout);
 
-require LWP::UserAgent;
-require HTTP::Request;
-require URI;
+use Scalar::Util 'blessed';
+use File::Temp;
+use IO::Handle;
+use Module::Load;
 
-use RPC::XML 'bytelength';
-require RPC::XML::Parser;
+use LWP::UserAgent;
+use HTTP::Request;
+use URI;
 
-$VERSION = do { my @r=(q$Revision: 1.22 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+use RPC::XML;
+use RPC::XML::ParserFactory;
+
+BEGIN
+{
+    # Check for compression support
+    $COMPRESSION_AVAILABLE =
+        (eval { load Compress::Zlib; 1; }) ? 'deflate' : q{};
+}
+
+$VERSION = '1.42';
+$VERSION = eval $VERSION; ## no critic (ProhibitStringyEval)
 
 ###############################################################################
 #
@@ -68,38 +81,41 @@ $VERSION = do { my @r=(q$Revision: 1.22 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r
 ###############################################################################
 sub new
 {
-    my $class = shift;
-    my $location = shift;
-    my %attrs = @_;
+    my ($class, $location, %attrs) = @_;
 
     $class = ref($class) || $class;
-    return "${class}::new: Missing location argument" unless $location;
+    if (! $location)
+    {
+        return "${class}::new: Missing location argument";
+    }
 
-    my ($self, $UA, $REQ, $PARSER);
+    my ($self, $UA, $REQ);
 
     # Start by getting the LWP::UA object
-    $UA = LWP::UserAgent->new((exists $attrs{useragent}) ?
-                              @{$attrs{useragent}} : ()) or
-        return "${class}::new: Unable to get LWP::UserAgent object";
-    $UA->agent(sprintf("%s/%s %s", $class, $VERSION, $UA->agent));
+    $UA = LWP::UserAgent->new(
+        (exists $attrs{useragent}) ? @{$attrs{useragent}} : ()
+    );
+    $UA->agent(sprintf '%s/%s %s', $class, $VERSION, $UA->agent);
     $self->{__useragent} = $UA;
     delete $attrs{useragent};
 
     # Next get the request object for later use
-    $REQ = HTTP::Request->new(POST => $location) or
-        return "${class}::new: Unable to get HTTP::Request object";
+    $REQ = HTTP::Request->new(POST => $location);
     $self->{__request} = $REQ;
     $REQ->header(Content_Type => 'text/xml');
     $REQ->protocol('HTTP/1.0');
 
-    # Check for compression support
-    $self->{__compress} = '';
-    eval "require Compress::Zlib";
-    $self->{__compress} = $@ ? '' : 'deflate';
+    # Note compression support
+    $self->{__compress} = $COMPRESSION_AVAILABLE;
     # It looks wasteful to keep using the hash key, but it makes it easier
-    # to change the string in just one place (above) if I have to.
-    $REQ->header(Accept_Encoding => $self->{__compress})
-        if $self->{__compress};
+    # to change the string in just one place (BEGIN block, above) if I have to.
+    # Also (for now) I prefer to manipulate the private keys directly, before
+    # blessing $self, rather than using accessors. This is just for performance
+    # and I might change my mind later.
+    if ($self->{__compress})
+    {
+        $REQ->header(Accept_Encoding => $self->{__compress});
+    }
     $self->{__compress_thresh} = $attrs{compress_thresh} || 4096;
     $self->{__compress_re} = qr/$self->{__compress}/;
     # They can change this value with a method
@@ -110,8 +126,8 @@ sub new
     # files due to size, and where to home the temp files. Start with a size
     # threshhold of 1Meg and no specific dir (which will fall-through to the
     # tmpdir() method of File::Spec).
-    $self->{__message_file_thresh} = $attrs{message_file_thresh} || 1048576;
-    $self->{__message_temp_dir}    = $attrs{message_temp_dir}    || '';
+    $self->{__message_file_thresh} = $attrs{message_file_thresh} || 1_048_576;
+    $self->{__message_temp_dir}    = $attrs{message_temp_dir}    || q{};
     delete @attrs{qw(message_file_thresh message_temp_dir)};
 
     # Note and preserve any error or fault handlers. Check the combo-handler
@@ -133,16 +149,19 @@ sub new
         delete $attrs{error_handler};
     }
 
-    # Get the RPC::XML::Parser instance
-    $self->{__parser} = RPC::XML::Parser->new($attrs{parser} ?
-                                              @{$attrs{parser}} : ()) or
-        return "${class}::new: Unable to get RPC::XML::Parser object";
+    # Get the RPC::XML::Parser instance from the ParserFactory
+    $self->{__parser} =
+        RPC::XML::ParserFactory->new($attrs{parser} ? @{$attrs{parser}} : ())
+              or return "${class}::new: Unable to get RPC::XML::Parser object";
     delete $attrs{parser};
 
     # Now preserve any remaining attributes passed in
-    $self->{$_} = $attrs{$_} for (keys %attrs);
+    for (keys %attrs)
+    {
+        $self->{$_} = $attrs{$_};
+    }
 
-    bless $self, $class;
+    return bless $self, $class;
 }
 
 ###############################################################################
@@ -167,17 +186,18 @@ sub simple_request
 {
     my ($self, @args) = @_;
 
-    my ($return, $value);
+    my $return;
 
-    $RPC::XML::ERROR = '';
+    $RPC::XML::ERROR = q{};
 
     $return = $self->send_request(@args);
-    unless (ref $return)
+    if (! ref $return)
     {
         $RPC::XML::ERROR = ref($self) . "::simple_request: $return";
-        return undef;
+        return;
     }
-    $return->value;
+
+    return $return->value;
 }
 
 ###############################################################################
@@ -190,32 +210,44 @@ sub simple_request
 #
 #   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
 #                   $self     in      ref       Class instance
-#                   $req      in      ref       RPC::XML::request object
+#                   $req      in      ref       RPC::XML::request object or
+#                                                 remote method name
+#                   @args     in      list      If $req is a method name, these
+#                                                 are potential arguments for
+#                                                 the remote call
 #
 #   Returns:        Success:    RPC::XML::response object instance
 #                   Failure:    error string
 #
 ###############################################################################
-sub send_request
+sub send_request ## no critic (ProhibitExcessComplexity)
 {
     my ($self, $req, @args) = @_;
 
     my ($me, $message, $response, $reqclone, $content, $can_compress, $value,
-        $do_compress, $req_fh, $tmpfile, $com_engine);
+        $do_compress, $req_fh, $tmpdir, $com_engine);
 
     $me = ref($self) . '::send_request';
 
-    if (! UNIVERSAL::isa($req, 'RPC::XML::request'))
+    if (! $req)
+    {
+        return "$me: No request object or remote method name given";
+    }
+    elsif (! (blessed $req and $req->isa('RPC::XML::request')))
     {
         # Assume that $req is the name of the routine to be called
-        $req = RPC::XML::request->new($req, @args);
-        return "$me: Error creating RPC::XML::request object: $RPC::XML::ERROR"
-            unless ($req); # $RPC::XML::ERROR is already set
+        if (! ($req = RPC::XML::request->new($req, @args)))
+        {
+            return "$me: Error creating RPC::XML::request object: " .
+                $RPC::XML::ERROR;
+        }
     }
 
     # Start by setting up the request-clone for using in this instance
     $reqclone = $self->request->clone;
-    $reqclone->header(Host => URI->new($reqclone->uri)->host);
+    if (! $reqclone->header('Host')) {
+        $reqclone->header(Host => URI->new($reqclone->uri)->host_port);
+    }
     $can_compress = $self->compress; # Avoid making 4+ calls to the method
     if ($self->compress_requests and $can_compress and
         $req->length >= $self->compress_thresh)
@@ -233,16 +265,17 @@ sub send_request
         $self->message_file_thresh <= $req->length)
     {
         require File::Spec;
-        require Symbol;
         # Start by creating a temp-file
-	$tmpfile = $self->message_temp_dir || File::Spec->tmpdir;
-        $tmpfile = File::Spec->catfile($tmpfile, __PACKAGE__ . $$ . time);
-        $req_fh = Symbol::gensym();
-        return "$me: Error opening $tmpfile: $!"
-            unless (open($req_fh, "+> $tmpfile"));
-        unlink $tmpfile;
+        $tmpdir = $self->message_temp_dir || File::Spec->tmpdir;
+        # File::Temp->new() croaks on error, rather than just returning undef
+        $req_fh = eval { File::Temp->new(UNLINK => 1, DIR => $tmpdir) };
+        if (! $req_fh)
+        {
+            return "$me: Error opening tmpfile: $@";
+        }
+        binmode $req_fh;
         # Make it auto-flush
-        my $old_fh = select($req_fh); $| = 1; select($old_fh);
+        $req_fh->autoflush();
 
         # Now that we have it, spool the request to it. This is a little
         # hairy, since we still have to allow for compression. And though the
@@ -251,64 +284,82 @@ sub send_request
         # into the primary handle.
         if ($do_compress && ($req->length >= $self->compress_thresh))
         {
-	    my $fh2 = Symbol::gensym();
-            $tmpfile .= '-2';
-            return "$me: Error opening $tmpfile: $!"
-                unless (open($fh2, "+> $tmpfile"));
-            unlink $tmpfile;
+            my $fh_compress = eval {
+                File::Temp->new(UNLINK => 1, DIR => $tmpdir);
+            };
+            if (! $fh_compress)
+            {
+                return "$me: Error opening compression tmpfile: $@";
+            }
             # Make it auto-flush
-            $old_fh = select($fh2); $| = 1; select($old_fh);
+            $fh_compress->autoflush();
 
             # Write the request to the second FH
-            $req->serialize($fh2);
-            seek($fh2, 0, 0);
+            $req->serialize($fh_compress);
+            seek $fh_compress, 0, 0;
 
             # Spin up the compression engine
             $com_engine = Compress::Zlib::deflateInit();
-            return "$me: Unable to initialize the Compress::Zlib engine"
-                unless $com_engine;
+            if (! $com_engine)
+            {
+                return "$me: Unable to initialize the Compress::Zlib engine";
+            }
 
             # Spool from the second FH through the compression engine, into
             # the intended FH.
-            my $buf = '';
+            my $buf = q{};
             my $out;
-            while (read($fh2, $buf, 4096))
+            while (read $fh_compress, $buf, 4096)
             {
                 $out = $com_engine->deflate(\$buf);
-                return "$me: Compression failure in deflate()"
-                    unless defined $out;
-                print $req_fh $out;
+                if (! defined $out)
+                {
+                    return "$me: Compression failure in deflate()";
+                }
+                print {$req_fh} $out;
             }
             # Make sure we have all that's left
             $out = $com_engine->flush;
-            return "$me: Compression flush failure in deflate()"
-                unless defined $out;
-            print $req_fh $out;
+            if (! defined $out)
+            {
+                return "$me: Compression flush failure in deflate()";
+            }
+            print {$req_fh} $out;
 
             # Close the secondary FH. Rewinding the primary is done later.
-            close($fh2);
+            if (! close $fh_compress)
+            {
+                return "$me: Error closing spool-file: $!";
+            }
         }
         else
         {
             $req->serialize($req_fh);
         }
-        seek($req_fh, 0, 0);
+        seek $req_fh, 0, 0;
 
         $reqclone->content_length(-s $req_fh);
         $reqclone->content(sub {
-                               my $b = '';
-                               return undef
-                                   unless defined(read($req_fh, $b, 4096));
-                               $b;
+                               my $b = q{};
+                               if (! defined read $req_fh, $b, 4096)
+                               {
+                                   return;
+                               }
+
+                               return $b;
                            });
     }
     else
     {
         # Treat the content strictly in-memory
-        $content = $req->as_string;
-        $content = Compress::Zlib::compress($content) if $do_compress;
+        utf8::encode($content = $req->as_string);
+        if ($do_compress)
+        {
+            $content = Compress::Zlib::compress($content);
+        }
         $reqclone->content($content);
-        $reqclone->content_length(bytelength($content));
+        # Because $content has been force-downgraded, length() should work
+        $reqclone->content_length(length $content);
     }
 
     # Content used to be handled as an in-memory string. Now, to avoid eating
@@ -319,72 +370,105 @@ sub send_request
     my $compression;
     my $parser = $self->parser->parse(); # Gets the ExpatNB object
     my $cb = sub {
-        my ($data, $resp) = @_;
+        my ($data_in, $resp) = @_;
 
-        unless (defined $compression)
+        if (! defined $compression)
         {
             $compression =
-                (($resp->content_encoding || '') =~
+                (($resp->content_encoding || q{}) =~
                  $self->compress_re) ? 1 : 0;
-            die "$me: Compressed content encoding not supported"
-                if ($compression and (! $can_compress));
+            if ($compression and (! $can_compress))
+            {
+                die "$me: Compressed content encoding not supported\n";
+            }
             if ($compression)
             {
-                die "$me: Unable to initialize de-compression engine"
-                    unless ($com_engine = Compress::Zlib::inflateInit());
+                if (! ($com_engine = Compress::Zlib::inflateInit()))
+                {
+                    die "$me: Unable to initialize de-compression engine\n";
+                }
             }
         }
 
         if ($compression)
         {
             my $error;
-            die "$me: Error in inflate() expanding data: $error"
-                unless (($data, $error) = $com_engine->inflate($data));
+            if (! (($data_in, $error) = $com_engine->inflate($data_in)))
+            {
+                die "$me: Error in inflate() expanding data: $error\n";
+            }
         }
 
-        $parser->parse_more($data);
+        $parser->parse_more($data_in);
         1;
     };
 
     $response = $self->useragent->request($reqclone, $cb);
     if ($message = $response->headers->header('X-Died'))
     {
+        $parser->release();
+
         # One of the die's was triggered
-        return (ref($self->error_handler) eq 'CODE') ?
+        return ('CODE' eq ref $self->error_handler) ?
             $self->error_handler->($message) : $message;
     }
-    unless ($response->is_success)
+    if (! $response->is_success)
     {
+        $parser->release();
+
         $message =  "$me: HTTP server error: " . $response->message;
-        return (ref($self->error_handler) eq 'CODE') ?
+        return ('CODE' eq ref $self->error_handler) ?
             $self->error_handler->($message) : $message;
     }
 
     # Whee. No errors from the callback or the server. Finalize the parsing
     # process.
-    eval { $value = $parser->parse_done(); };
-    if ($@)
+    if (! eval { $value = $parser->parse_done(); 1; })
     {
-        # One of the die's was triggered
-        return (ref($self->error_handler) eq 'CODE') ?
-            $self->error_handler->($@) : $@;
+        if ($@)
+        {
+            # One of the die's was triggered
+            return ('CODE' eq ref $self->error_handler) ?
+                $self->error_handler->($@) : $@;
+        }
     }
 
     # Check if there is a callback to be invoked in the case of
     # errors or faults
-    if (! ref($value))
+    if (! ref $value)
     {
         $message =  "$me: parse-level error: $value";
-        return (ref($self->error_handler) eq 'CODE') ?
+        return ('CODE' eq ref $self->error_handler) ?
             $self->error_handler->($message) : $message;
     }
     elsif ($value->is_fault)
     {
-        return (ref($self->fault_handler) eq 'CODE') ?
+        return ('CODE' eq ref $self->fault_handler) ?
             $self->fault_handler->($value->value) : $value->value;
     }
 
-    $value->value;
+    return $value->value;
+}
+
+###############################################################################
+#
+#   Sub Name:       timeout
+#
+#   Description:    Get or set the timeout() setting on the underlying
+#                   LWP::UserAgent object.
+#
+#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
+#                   $self     in      ref       Object of this class
+#                   $time     in      scalar    New timeout value, if passed
+#
+#   Returns:        Return value from LWP::UserAgent->timeout()
+#
+###############################################################################
+sub timeout ## no critic (RequireArgUnpacking)
+{
+    my $self = shift;
+
+    return $self->useragent->timeout(@_);
 }
 
 ###############################################################################
@@ -400,10 +484,11 @@ sub send_request
 #   Returns:        Current URI, undef if trying to set an invalid URI
 #
 ###############################################################################
-sub uri
+sub uri ## no critic (RequireArgUnpacking)
 {
     my $self = shift;
-    $self->request->uri(@_);
+
+    return $self->request->uri(@_);
 }
 
 ###############################################################################
@@ -428,17 +513,18 @@ sub credentials
 
     my $uri = URI->new($self->uri);
     $self->useragent->credentials($uri->host_port, $realm, $user, $pass);
-    $self;
+
+    return $self;
 }
 
 # Immutable accessor methods
 BEGIN
 {
-    no strict 'refs';
+    no strict 'refs'; ## no critic (ProhibitNoStrict)
 
     for my $method (qw(useragent request compress_re compress parser))
     {
-        *$method = sub { shift->{"__$method"} }
+        *{$method} = sub { shift->{"__$method"} }
     }
 }
 
@@ -446,22 +532,28 @@ BEGIN
 sub compress_thresh
 {
     my $self = shift;
-    my $set = shift || 0;
+    my $value = shift || 0;
 
     my $old = $self->{__compress_thresh};
-    $self->{__compress_thresh} = $set if ($set);
+    if ($value)
+    {
+        $self->{__compress_thresh} = $value;
+    }
 
-    $old;
+    return $old;
 }
 
 # This doesn't actually *get* the original value, it only sets the value
 sub compress_requests
 {
-    my $self = shift;
+    my ($self, $value) = @_;
 
-    return $self->{__compress_requests} unless @_;
+    if (! $value)
+    {
+        return $self->{__compress_requests};
+    }
 
-    $self->{__compress_requests} = $_[0] ? 1 : 0;
+    return $self->{__compress_requests} = $value ? 1 : 0;
 }
 
 # These are get/set accessors for the fault-handler, error-handler and the
@@ -471,48 +563,68 @@ sub fault_handler
     my ($self, $newval) = @_;
 
     my $val = $self->{__fault_cb};
-    $self->{__fault_cb} = $newval if ($newval and ref($newval));
+    if ($newval and ref $newval)
+    {
+        $self->{__fault_cb} = $newval;
+    }
     # Special: an explicit undef is used to clear the callback
-    $self->{__fault_cb} = undef if (@_ == 2 and (! defined $newval));
+    if (@_ == 2 and (! defined $newval))
+    {
+        $self->{__fault_cb} = undef;
+    }
 
-    $val;
+    return $val;
 }
+
 sub error_handler
 {
     my ($self, $newval) = @_;
 
     my $val = $self->{__error_cb};
-    $self->{__error_cb} = $newval if ($newval and ref($newval));
+    if ($newval and ref $newval)
+    {
+        $self->{__error_cb} = $newval;
+    }
     # Special: an explicit undef is used to clear the callback
-    $self->{__error_cb} = undef if (@_ == 2 and (! defined $newval));
+    if (@_ == 2 and (! defined $newval))
+    {
+        $self->{__error_cb} = undef;
+    }
 
-    $val;
+    return $val;
 }
+
 sub combined_handler
 {
     my ($self, $newval) = @_;
 
-    ($self->fault_handler($newval), $self->error_handler($newval));
+    return ($self->fault_handler($newval), $self->error_handler($newval));
 }
 
 # Control whether, and at what point, messages are considered too large to
 # handle in-memory.
 sub message_file_thresh
 {
-    my $self = shift;
+    my ($self, $thresh) = @_;
 
-    return $self->{__message_file_thresh} unless @_;
+    if (! $thresh)
+    {
+        return $self->{__message_file_thresh};
+    }
 
-    $self->{__message_file_thresh} = shift;
+    return $self->{__message_file_thresh} = $thresh;
 }
 
 sub message_temp_dir
 {
-    my $self = shift;
+    my ($self, $dir) = @_;
 
-    return $self->{__message_temp_dir} unless @_;
+    if (! $dir)
+    {
+        return $self->{__message_temp_dir};
+    }
 
-    $self->{__message_temp_dir} = shift;
+    return $self->{__message_temp_dir} = $dir;
 }
 
 1;
@@ -541,7 +653,7 @@ This is an XML-RPC client built upon the B<RPC::XML> data classes, and using
 B<LWP::UserAgent> and B<HTTP::Request> for the communication layer. This
 client supports the full XML-RPC specification.
 
-=head1 METHODS
+=head1 SUBROUTINES/METHODS
 
 The following methods are available:
 
@@ -564,11 +676,11 @@ treated specially:
 
 =item parser
 
-If this parameter is passed, the value following it is expected to be an
-array reference. The contents of that array are passed to the B<new> method
-of the B<RPC::XML::Parser> object that the client object caches for its use.
-See the B<RPC::XML::Parser> manual page for a list of recognized parameters
-to the constructor.
+If this parameter is passed, the value following it is expected to be an array
+reference. The contents of that array are passed to the B<new> method of the
+B<RPC::XML::ParserFactory>-generated object that the client object caches for
+its use. See the B<RPC::XML::ParserFactory> manual page for a list of
+recognized parameters to the constructor.
 
 =item useragent
 
@@ -612,7 +724,7 @@ temporary file, and spooled from there instead. This is useful for cases in
 which the request includes B<RPC::XML::base64> objects that are themselves
 spooled from file-handles. This test is independent of compression, so even
 if compression of a request would drop it below this threshhold, it will be
-spooled anyway. The file itself is unlinked after the file-handle is created,
+spooled anyway. The file itself is created via File::Temp with UNLINK=>1,
 so once it is freed the disk space is immediately freed.
 
 =item message_temp_dir
@@ -689,6 +801,14 @@ client object to a different (protected) location would require calling this
 with new credentials for the new realm (even if the value of C<$realm> is
 identical at both locations).
 
+=item timeout ([INTEGER])
+
+Get or set the current time-out value on the underlying B<LWP::UserAgent>
+object that this object uses for sending requests. This is just a proxy
+through to the method of the same name in the B<LWP::UserAgent> class. The
+return value is the current time-out value (prior to change, if a new value
+is given).
+
 =item message_file_thresh
 
 =item message_temp_dir
@@ -755,24 +875,64 @@ readability of the code took precedence over general efficiency. It is now
 being maintained as production code, but may still have parts that could be
 written more efficiently.
 
+=head1 BUGS
+
+Please report any bugs or feature requests to
+C<bug-rpc-xml at rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=RPC-XML>. I will be
+notified, and then you'll automatically be notified of progress on
+your bug as I make changes.
+
+=head1 SUPPORT
+
+=over 4
+
+=item * RT: CPAN's request tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=RPC-XML>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/RPC-XML>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/d/RPC-XML>
+
+=item * Search CPAN
+
+L<http://search.cpan.org/dist/RPC-XML>
+
+=item * MetaCPAN
+
+L<https://metacpan.org/release/RPC-XML>
+
+=item * Source code on GitHub
+
+L<http://github.com/rjray/rpc-xml>
+
+=back
+
+=head1 LICENSE AND COPYRIGHT
+
+This file and the code within are copyright (c) 2011 by Randy J. Ray.
+
+Copying and distribution are permitted under the terms of the Artistic
+License 2.0 (L<http://www.opensource.org/licenses/artistic-license-2.0.php>) or
+the GNU LGPL 2.1 (L<http://www.opensource.org/licenses/lgpl-2.1.php>).
+
 =head1 CREDITS
 
 The B<XML-RPC> standard is Copyright (c) 1998-2001, UserLand Software, Inc.
 See <http://www.xmlrpc.com> for more information about the B<XML-RPC>
 specification.
 
-=head1 LICENSE
-
-This module is licensed under the terms of the Artistic License that covers
-Perl. See <http://www.opensource.org/licenses/artistic-license.php> for the
-license itself.
-
 =head1 SEE ALSO
 
-L<RPC::XML>, L<RPC::XML::Server>
+L<RPC::XML|RPC::XML>, L<RPC::XML::Server|RPC::XML::Server>
 
 =head1 AUTHOR
 
-Randy J. Ray <rjray@blackperl.com>
+Randy J. Ray C<< <rjray@blackperl.com> >>
 
 =cut
